 matrix   World;				//World Matrix
 matrix   WorldViewProjection;	//WorldViewProjection Matrix
 
 float3   CamPos;               //Camera Position

 float4	  Ambient;				//Light - Ambient Color
 float    AIntensity;			//Light - Ambient Intensity
 float4   Diffuse;				//Light - Diffuse Color
 float    DIntensity; 			//Light - Diffuse Intensity
 float3   LightDirection;       //Light - Light Direction

 float4   Specular;				//Material - Specular Color
 float    SIntensity;			//Material - Specular Intensity
 float    Sharpen;				//Material - Specular Power
 float    Brightness;			//Material - Light Power
 float    HeightMapScale;       //Material - Heightmap Scaling

 Texture  ColorMap;				//ColorMap
 Texture  NormalMap;			//NormalMap
 Texture  AmbDisSpecMap;		//AmbDisSpecMap


// -----------------------------------------------------------------------------------------------------------------
//  Sampler
// -----------------------------------------------------------------------------------------------------------------

//ColorMap
sampler ColorMapSampler=sampler_state 	  { texture   = <ColorMap>;
		 									minfilter = LINEAR;
		 									magfilter = LINEAR;
		 									mipfilter = LINEAR;
		 									AddressU  = Wrap;
		 									AddressV  = Wrap;};
//NormalMap
sampler NormalMapSampler=sampler_state 	  { texture   = <NormalMap>;
									     	minfilter = LINEAR;
									     	magfilter = LINEAR;
									     	mipfilter = LINEAR;
									    	AddressU  = Wrap;
									     	AddressV  = Wrap;};
//AmbDisSpecMap
sampler AmbDisSpecMapSampler=sampler_state{ texture   = <AmbDisSpecMap>;
									     	minfilter = LINEAR;
									     	magfilter = LINEAR;
									    	mipfilter = LINEAR;
									     	AddressU  = Wrap;
									     	AddressV  = Wrap;};
//ShadowMap
sampler ShadowMapSampler = sampler_state  { Texture = <ShadowMap>;
    									    minfilter = POINT;
    									    magfilter = POINT;
   										    mipfilter = NONE;
    									    AddressU  = Clamp;
   										    AddressV  = Clamp;};

// -----------------------------------------------------------------------------------------------------------------
//  Structures
// -----------------------------------------------------------------------------------------------------------------

//Vertex Output Structure Color
struct vs_OutCol {
   float4 pos				: POSITION;			//Position in ClipSpace	
   float3 l     		    : TEXCOORD1;		//LightVector
   float3 n		            : TEXCOORD2;		//Normal
   float3 h                 : TEXCOORD3;		//HalfVector
};

//Vertex Output Structure Texture
struct vs_OutTex {
   float4 pos				: POSITION;			//Position in ClipSpace
   float3 v					: TEXCOORD0;		//ViewVector
   float3 l     		    : TEXCOORD1;		//LightVector
   float3 n		            : TEXCOORD2;		//Normal
   float3 h                 : TEXCOORD3;		//HalfVector
   float2 tex				: TEXCOORD4;		//Texture Coords
};


// -----------------------------------------------------------------------------------------------------------------
// VertexShader 
// -----------------------------------------------------------------------------------------------------------------

// VertexShader Color
// ------------------
vs_OutCol vsoutCOL(float4 pos : POSITION, 
		           float3 N   : NORMAL)
{
  vs_OutCol Out;

  float3 posworld = mul(pos,World).xyz;			//Position in Worldspace
  Out.pos   = mul(pos,WorldViewProjection); 	//Out - Position in ClipSpace
  Out.l		= normalize(LightDirection);        //Out - normalized LightVector
  Out.h     = normalize(Out.l + (CamPos - posworld.xyz)); 
  Out.n     = mul(N,World).xyz;					//Out - normalized Normals from Model
  return Out;
}


// VertexShader Textured
// ---------------------
vs_OutTex vsoutTEX(float4 pos : POSITION, 
		           float3 N   : NORMAL, 
		           float2 tex : TEXCOORD0)
{
  vs_OutTex Out;

  float3 posworld = mul(pos,World).xyz;			//Position in Worldspace

  Out.pos   = mul(pos,WorldViewProjection); 	//Out - Position in ClipSpace
  Out.tex   = tex;		                        //Out - TextureCoords
  Out.v  	= normalize(CamPos - posworld.xyz); //Out - normalized ViewVector
  Out.l		= normalize(LightDirection);        //Out - normalized LightVector
  Out.h     = normalize(Out.l + Out.v); 
  Out.n     = mul(N,World).xyz;					//Out - normalized Normals from Model
  return Out;
}


// VertexShader Textured and Normal
// --------------------------------
vs_OutTex vsoutTBN(float4 pos : POSITION, 
		           float3 N   : NORMAL, 
		           float2 tex : TEXCOORD0, 
		           float4 T   : TANGENT , 
		           float4 B   : BINORMAL)
{
  vs_OutTex Out;
  float3x3 WorldToTangentSpace;					//World to Tangent Matrix
  float3   lightvec;							//Normalized Lightvector in Tangentspace
  float3   posworld;							//Position in Worldspace
  float3   viewvec;								//Normalized ViewVector in Tangentspace
  float3   halfvec;

  float3 b = normalize(mul(T,World));			//B - Tangent
  float3 t = normalize(mul(B,World));	        //T - Binormal
  float3 n = normalize(mul(N,World));			//N - Normal

  WorldToTangentSpace[0]=t;						//T - Tangent  to TangentSpace
  WorldToTangentSpace[1]=b;	        			//B - Binormal to TangentSpace
  WorldToTangentSpace[2]=n;						//N - Normal   to TangentSpace

  posworld      = mul(pos,World).xyz;						   //Position in WorldSpace
  viewvec       = mul(WorldToTangentSpace,(CamPos - posworld));//ViewVector
  lightvec      = mul(WorldToTangentSpace,LightDirection);     //LightVector
  halfvec       = lightvec + viewvec;

  Out.pos = mul(pos,WorldViewProjection); 		//Out - Position in ClipSpace
  Out.tex = tex;								//Out - TextureCoords
  Out.v   = normalize(viewvec); 				//Out - ViewVector
  Out.l   = normalize(lightvec);				//Out - normalized LightVector in TangentSpace
  Out.h   = normalize(halfvec);
  Out.n   = N;									//Out - Normals from Model
  return Out;
}









// -----------------------------------------------------------------------------------------------------------------
//  PixelShader Section Light without Shadowmapping
// -----------------------------------------------------------------------------------------------------------------

// PixelShader Color
// ----------------------
float4 psoutCol0(float3 v : TEXCOORD0, 
				 float3 l : TEXCOORD1, 
				 float3 n : TEXCOORD2, 
				 float3 h : TEXCOORD3) : COLOR
{
  float  nDotL = saturate(dot(n, l));							//Dot(Normal,Light) for Diffuse
  float  nDotH = saturate(dot(n, h));							//Dot(Normal,Half) for Specular
  float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);         //Calculate SpecularPower

  float4 lAmbient  = Ambient * AIntensity;					  	//New Color from Ambient
  float4 lDiffuse  = Diffuse * DIntensity  * nDotL; 		   	//New Color from Diffuse
  float4 lSpecular = Specular* SIntensity  * power * nDotL;		//New Color from Specular
  float4 light     = saturate(lAmbient + lDiffuse);
  return saturate (light + lSpecular)* Brightness;
}

//  PixelShader Colored no Spec
// ----------------------------
float4 psoutCol1(float3 v : TEXCOORD0, 
	             float3 l : TEXCOORD1, 
	             float3 n : TEXCOORD2, 
	             float3 h : TEXCOORD3) : COLOR
{
  float  nDotL = saturate(dot(n, l));						  //Dot(Normal,Light) for Diffuse

  float4 lAmbient  = Ambient * AIntensity;					  //New Color from Ambient
  float4 lDiffuse  = Diffuse * DIntensity  * nDotL; 		  //New Color from Diffuse

  return saturate (lAmbient + lDiffuse)* Brightness;
}
// - End -



//  PixelShader Textured
// ---------------------
float4 psoutTex0(float3 v   : TEXCOORD0, 
				 float3 l   : TEXCOORD1, 
				 float3 n   : TEXCOORD2, 
				 float3 h   : TEXCOORD3, 
				 float2 tex : TEXCOORD4) : COLOR
{
  float  nDotL = saturate(dot(n, l));									 //Dot(Normal,Light) for Diffuse
  float  nDotH = saturate(dot(n, h));									 //Dot(Normal,Half) for Specular
  float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);                  //Calculate SpecularPower

  float4 lColor    = tex2D(ColorMapSampler, tex);						 //New Color from ColorMap
  float4 lAmbient  = Ambient * AIntensity;					  			 //New Color from Ambient
  float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		     //New Color from Diffuse
  float4 lSpecular = saturate(Specular * SIntensity  * power);  		 //New Color from Specular
  float4 light     = saturate(lAmbient + lDiffuse);
  return saturate (light * lColor + lSpecular)* Brightness;
}

//  PixelShader Textured no Spec
// -----------------------------
float4 psoutTex1(float3 v   : TEXCOORD0, 
                 float3 l   : TEXCOORD1, 
                 float3 n   : TEXCOORD2, 
                 float3 h   : TEXCOORD3, 
                 float2 tex : TEXCOORD4) : COLOR
{
  float  nDotL = saturate(dot(n, l));									 //Dot(Normal,Light) for Diffuse

  float4 lColor    = tex2D(ColorMapSampler, tex);						 //New Color from ColorMap
  float4 lAmbient  = Ambient * AIntensity;					  			 //New Color from Ambient
  float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		     //New Color from Diffuse
  float4 light     = saturate(lAmbient + lDiffuse);
  return saturate (light * lColor)* Brightness;
}
// - End -



//  PixelShader TexturedADS
// ------------------------
float4 psoutTexAS0(float3 v   : TEXCOORD0, 
				   float3 l   : TEXCOORD1, 
				   float3 n   : TEXCOORD2, 
				   float3 h   : TEXCOORD3, 
				   float2 tex : TEXCOORD4) : COLOR
{
  float2 AmDisSp = tex2D(AmbDisSpecMapSampler,tex).rb;		
  
  float  nDotL = saturate(dot(n, l));									       
  float  nDotH = saturate(dot(n, h));									   
  float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);                  

  float4 lColor    = tex2D(ColorMapSampler, tex);						  
  float4 lAmbient  = Ambient * AIntensity * AmDisSp.r;					  			  
  float4 lDiffuse  = Diffuse * DIntensity * nDotL * AmDisSp.r; 		   	  		       
  float4 light     = saturate(lAmbient + lDiffuse);
  float4 lSpecular = saturate(Specular* SIntensity * power)* AmDisSp.g; 
  
  return saturate (light * lColor + lSpecular)* Brightness;
}

//  PixelShader TexturedADS no Spec
// --------------------------------
float4 psoutTexAS1(float3 v   : TEXCOORD0, 
	               float3 l   : TEXCOORD1, 
				   float3 n   : TEXCOORD2, 
				   float3 h   : TEXCOORD3, 
				   float2 tex : TEXCOORD4) : COLOR
{
  float2 AmDisSp = tex2D(AmbDisSpecMapSampler,tex).r;		
  float  nDotL = saturate(dot(n, l));									       								   

  float4 lColor    = tex2D(ColorMapSampler, tex);						  
  float4 lAmbient  = Ambient * AIntensity * AmDisSp.r;					  			  
  float4 lDiffuse  = Diffuse * DIntensity * nDotL * AmDisSp.r; 		   	  		       
  float4 light     = saturate(lAmbient + lDiffuse);
  return saturate (light * lColor)* Brightness;
}
// - End -



//  PixelShader Textured + Normalmap
// ---------------------------------
float4 psoutTexN0(float3 v   : TEXCOORD0, 
                  float3 l   : TEXCOORD1,
                  float3 n   : TEXCOORD2, 
                  float3 h   : TEXCOORD3, 
                  float2 tex : TEXCOORD4) : COLOR
{
  n=  tex2D(NormalMapSampler, tex).rgb*2-1; 				            //Normals from Map in Tangentspace

  float  nDotL = saturate(dot(n, l));									
  float  nDotH = saturate(dot(n, h));									
  float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);                 

  float4 lColor    = tex2D(ColorMapSampler, tex);						 
  float4 lAmbient  = Ambient * AIntensity ;					  			   
  float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		       
  float4 lSpecular = saturate(Specular* SIntensity * power);  
  float4 light     = saturate(lAmbient + lDiffuse);

  return saturate (light * lColor + lSpecular )* Brightness;
}

//  PixelShader Textured + Normalmap no Spec
// -----------------------------------------
float4 psoutTexN1(float3 v   :TEXCOORD0, 
                  float3 l   :TEXCOORD1, 
                  float3 n   :TEXCOORD2, 
                  float3 h   :TEXCOORD3, 
                  float2 tex :TEXCOORD4) : COLOR
{
  n=  tex2D(NormalMapSampler, tex).rgb*2-1; 				            //Normals from Map in Tangentspace
  float  nDotL = saturate(dot(n, l));									

  float4 lColor    = tex2D(ColorMapSampler, tex);						
  float4 lAmbient  = Ambient * AIntensity ;					  			  
  float4 lDiffuse  = Diffuse * DIntensity * nDotL; 		   	  		       
  float4 light     = saturate(lAmbient + lDiffuse);

  return saturate (light * lColor )* Brightness;
}
// - End -



//  PixelShader  TexturedADS + Normalmap
// -------------------------------------
float4 psoutTexNAS0(float3 v   : TEXCOORD0, 
                    float3 l   : TEXCOORD1,
                    float3 n   : TEXCOORD2, 
                    float3 h   : TEXCOORD3, 
                    float2 tex : TEXCOORD4) : COLOR
{
               n = tex2D(NormalMapSampler, tex).rgb*2-1; 				          //Normals from Map in Tangentspace
  float2 AmDisSp = tex2D(AmbDisSpecMapSampler,tex).rb;

  float  nDotL = saturate(dot(n, l));									
  float  nDotH = saturate(dot(n, h));									
  float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);                 

  float4 lColor    = tex2D(ColorMapSampler, tex);						 
  float4 lAmbient  = Ambient * AIntensity * AmDisSp.r;					  			   
  float4 lDiffuse  = Diffuse * DIntensity * nDotL * AmDisSp.r; 		   	  		       
  float4 lSpecular = saturate(Specular* SIntensity * power)* AmDisSp.g;  
  float4 light     = saturate(lAmbient + lDiffuse);

  return saturate (light * lColor + lSpecular )* Brightness;
}

//  PixelShader TexturedADS + Normalmap no Spec
// --------------------------------------------
float4 psoutTexNAS1(float3 v   : TEXCOORD0, 
                    float3 l   : TEXCOORD1, 
                    float3 n   : TEXCOORD2, 
                    float3 h   : TEXCOORD3, 
                    float2 tex : TEXCOORD4) : COLOR
{
  n=  tex2D(NormalMapSampler, tex).rgb*2-1; 				            //Normals from Map in Tangentspace
  float AmDisSp = tex2D(AmbDisSpecMapSampler,tex).r;
  float  nDotL = saturate(dot(n, l));									

  float4 lColor    = tex2D(ColorMapSampler, tex);						
  float4 lAmbient  = Ambient * AIntensity *AmDisSp;					  			  
  float4 lDiffuse  = Diffuse * DIntensity * nDotL *AmDisSp; 		   	  		       
  float4 light     = saturate(lAmbient + lDiffuse);

  return saturate (light * lColor )* Brightness;
}
// - End -



//  PixelShader TexturedADS + Normalmap + Heightmap (Parallax)
// -----------------------------------------------------------
float4 psoutTexNADS0( float3 v   : TEXCOORD0,
                      float3 l   : TEXCOORD1,
                      float3 n   : TEXCOORD2,
                      float3 h   : TEXCOORD3,
                      float2 tex : TEXCOORD4) : COLOR
{
  float  height;
  float  dis    = tex2D(AmbDisSpecMapSampler,tex).g *2 -1;
  	     height = dis * HeightMapScale;
  	     tex    = tex + height * v.yx;
  		 n		= tex2D(NormalMapSampler,tex).rgb *2 -1; 
  float2 AmDisSp= tex2D(AmbDisSpecMapSampler,tex).rb;
  
  float  nDotL = saturate(dot(n,l));									   		 
  float  nDotH = saturate(dot(n,h));								   
  float  power = (nDotL == 0) ? 0 : pow(nDotH,Sharpen);             

  float4 lColor    = tex2D(ColorMapSampler, tex);						
  float4 lAmbient  = Ambient * AIntensity * AmDisSp.r;					  			
  float4 lDiffuse  = Diffuse * DIntensity * nDotL * AmDisSp.r ; 		   	  		    
  float4 lSpecular = saturate(Specular* SIntensity * power)* AmDisSp.g ;			
  float4 light     = saturate(lAmbient + lDiffuse);
  
  return saturate (light * lColor + lSpecular) * Brightness;
}

//  PixelShader TexturedADS + Normalmap + Heightmap (Parallax) no Spec
// -------------------------------------------------------------------
float4 psoutTexNADS1( float3 v   : TEXCOORD0,
                      float3 l   : TEXCOORD1, 
                      float3 n   : TEXCOORD2, 
                      float3 h   : TEXCOORD3,
                      float2 tex : TEXCOORD4) : COLOR
{
  float  height;
  float  dis    = tex2D(AmbDisSpecMapSampler,tex).g *2 -1;
  	     height = dis * HeightMapScale;
  	     tex    = tex + height * v.yx;
  		 n		= tex2D(NormalMapSampler,tex).rgb *2 -1; 
  
  float  AmDisSp= tex2D(AmbDisSpecMapSampler,tex).r;
  
  float  nDotL = saturate(dot(n,l));
  								   		 
  float4 lColor    = tex2D(ColorMapSampler, tex);						
  float4 lAmbient  = Ambient * AIntensity * AmDisSp.r;					  			
  float4 lDiffuse  = Diffuse * DIntensity * nDotL * AmDisSp.r ; 		   	  		    		
  float4 light     = saturate(lAmbient + lDiffuse);
  
  return saturate (light * lColor) * Brightness;
}
// - End -




// ---------------------------------------------------------------------
//  Technique Section Light
// ---------------------------------------------------------------------


technique DirLight
{
   pass pass0
   {
      VertexShader = compile vs_2_0 vsoutCOL();
      PixelShader  = compile ps_2_0 psoutCol0();
   }
   pass pass1
   {
      VertexShader = compile vs_2_0 vsoutCOL();
      PixelShader  = compile ps_2_0 psoutCol1();
   }
}
technique DirLightTex
{
   pass pass0
   {
      VertexShader = compile vs_2_0 vsoutTEX();
      PixelShader  = compile ps_2_0 psoutTex0();
   }
   pass pass1
   {
      VertexShader = compile vs_2_0 vsoutTEX();
      PixelShader  = compile ps_2_0 psoutTex1();
   }
}
technique DirLightTexAmbSpec
{
   pass pass0
   {
      VertexShader = compile vs_2_0 vsoutTEX();
      PixelShader  = compile ps_2_0 psoutTexAS0();
   }
   pass pass1
   {
      VertexShader = compile vs_2_0 vsoutTEX();
      PixelShader  = compile ps_2_0 psoutTexAS1();
   }
}
technique DirLightTexNrm
{
   pass pass0
   {
      VertexShader = compile vs_2_0 vsoutTBN();
      PixelShader  = compile ps_2_0 psoutTexN0();
   }
   pass pass1
   {
      VertexShader = compile vs_2_0 vsoutTBN();
      PixelShader  = compile ps_2_0 psoutTexN1();
   }
}
technique DirLightTexNrmAmbSpec
{
   pass pass0
   {
      VertexShader = compile vs_2_0 vsoutTBN();
      PixelShader  = compile ps_2_0 psoutTexNAS0();
   }
   pass pass1
   {
      VertexShader = compile vs_2_0 vsoutTBN();
      PixelShader  = compile ps_2_0 psoutTexNAS1();
   }
}
technique DirLightTexNrmAmbDisSpec
{
   pass pass0
   {
      VertexShader = compile vs_2_0 vsoutTBN();
      PixelShader  = compile ps_2_0 psoutTexNADS0();
   }
   pass pass1
   {
      VertexShader = compile vs_2_0 vsoutTBN();
      PixelShader  = compile ps_2_0 psoutTexNADS1();
   }
}
